from django.http import JsonResponse

from rest_framework import status

from django_resaas.models.front_end import FrontEnd
from django_resaas.core.utils.translate import Translate
from django_resaas.core.conf import get_setting



class FrontEndMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):

        path = request.path
        scope = self.get_url_scope(path)

        # URLs públicas globais
        if scope in get_setting('FRONT_END.PUBLIC_URL', []):
            return self.get_response(request)

        if str(get_setting('FRONT_END.REQUIRE_CREDENTIALS')).lower() in ['true', '1', 'yes']:

            fek = request.headers.get('FEK')
            fep = request.headers.get('FEP')

            if not fek or not fep:
                return self._unauthorized('Nao autorizado')

            frontend = FrontEnd.objects.filter(fek=fek, fep=fep).first()

            if not frontend:
                return self._unauthorized('Bad Credentials')

            # guardar no request
            request.frontend = frontend

            # 🔐 validar URL
            if not self._has_url_permission(frontend, scope):
                return self._forbidden('Sem permissão para esta rota')

            # 🔐 validar método HTTP
            if not self._has_method_permission(frontend, request.method):
                return self._forbidden('Sem permissão para esta operação')

        return self.get_response(request)

    # --------------------
    # Helpers
    # --------------------

    def get_url_scope(self, path):
        parts = path.strip('/').split('/')
        if len(parts) >= 2 and parts[0] == 'api':
            return parts[1]
        return None

    def _has_url_permission(self, frontend, scope):
        rules = get_setting('FRONT_END.URL_RULES', {})

        if not scope or scope not in rules:
            return True  # se não houver regra, permite

        return frontend.access in rules[scope]

    def _has_method_permission(self, frontend, method):
        method = method.upper()

        if frontend.access == 'super':
            return True  # 🔥 tudo permitido

        if frontend.access == 'read':
            return method in ['GET', 'HEAD', 'OPTIONS']

        if frontend.access == 'readwrite':
            return method in ['GET', 'POST', 'PUT', 'PATCH', 'HEAD', 'OPTIONS']

        if frontend.access == 'write':
            return method in ['POST', 'PUT', 'PATCH', 'DELETE', 'OPTIONS']

        return False

    def _unauthorized(self, msg):
        return JsonResponse(
            {
                'code': 10001,
                'alert_error': Translate.tdc(None, msg)
            },
            status=status.HTTP_401_UNAUTHORIZED
        )

    def _forbidden(self, msg):
        return JsonResponse(
            {
                'code': 10003,
                'alert_error': Translate.tdc(None, msg)
            },
            status=status.HTTP_403_FORBIDDEN
        )




import uuid

from django.db import models
from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.core.base.models import TimeModel
from django.contrib.contenttypes.models import ContentType


class TipoEntidadeModelo(TimeModel):
    tipo_entidade = models.ForeignKey(TipoEntidade, on_delete=models.CASCADE)
    modelo = models.ForeignKey(ContentType, on_delete=models.CASCADE)

    class Meta:
        permissions = ()

    def __str__(self):
        return f'{self.tipo_entidade.nome} | {self.modelo.name}'


# Generated by Django 5.2.10 on 2026-03-07 16:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_resaas', '0002_layoutsetting_entidade_layout_settings_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='layoutsetting',
            options={'verbose_name': 'LayoutSetting', 'verbose_name_plural': 'LayoutSettings'},
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='accent',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='dark',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='info',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='negative',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='positive',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='primary',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='secondary',
        ),
        migrations.RemoveField(
            model_name='layoutsetting',
            name='warning',
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='animations',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='button_dense',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='button_round',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='button_style',
            field=models.CharField(choices=[('flat', 'Flat'), ('outline', 'Outline'), ('unelevated', 'Unelevated'), ('push', 'Push')], default='unelevated', max_length=20),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='card_bordered',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='card_flat',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='dark_mode',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='dense',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='density',
            field=models.CharField(choices=[('dense', 'Dense'), ('normal', 'Normal'), ('comfortable', 'Comfortable')], default='normal', max_length=20),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='elevated',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='input_dense',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='input_style',
            field=models.CharField(choices=[('outlined', 'Outlined'), ('filled', 'Filled'), ('standout', 'Standout')], default='outlined', max_length=20),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='nome',
            field=models.CharField(default=None, max_length=100),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='ripple',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='rounded',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='sidebar_mini',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='sidebar_width',
            field=models.IntegerField(default=260),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='square',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='toolbar_dense',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='layoutsetting',
            name='toolbar_elevated',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='theme',
            name='accent',
            field=models.CharField(default='#9C250B0', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='dark',
            field=models.CharField(default='#1d1d1d', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='info',
            field=models.CharField(default='#31CCEC', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='negative',
            field=models.CharField(default='#C10015', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='positive',
            field=models.CharField(default='#21BA45', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='primary',
            field=models.CharField(default='#19506D2', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='secondary',
            field=models.CharField(default='#26A69A', max_length=50),
        ),
        migrations.AlterField(
            model_name='theme',
            name='warning',
            field=models.CharField(default='#F2C0350', max_length=50),
        ),
    ]



from django_resaas.core.base.serializers import BaseSerializer
from django_resaas.models.entidade import Entidade


class EntidadeGravarSerializer(BaseSerializer):
    permanent_fields_files = ['logo']

    class Meta:
        model = Entidade
        fields = [
            'id',
            'nome',
            'logo',
            'groups',
            'admins',
            'modelos',
            'display_logo',
            'display_qr',
            'display_bar',
            'tipo_entidade',
            'disc_space',
            'disc_used_space',
            'disc_free_space',
            'rodape',
            'estado',
        ]



from rest_framework import filters
from rest_framework import viewsets


from django_resaas.models.theme import Theme
from django_resaas.data.theme.serializers.theme import ThemeSerializer


class ThemeAPIView(viewsets.ModelViewSet):
    filter_backends = (filters.SearchFilter,)
    serializer_class = ThemeSerializer
    queryset = Theme.objects.all()

    def get_queryset(self):
        return self.queryset.filter()


from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.core.mail import EmailMultiAlternatives
from django.urls import reverse
from django.utils.encoding import smart_bytes
from django.utils.http import urlsafe_base64_encode

from rest_framework import generics, status
from rest_framework.response import Response

from django_resaas.models.user import User
from django_resaas.data.user.serializers.request_password_reset_email import (
    ResetPasswordEmailRequestSerializer
)
from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils.templates import render_email_template


class RequestPasswordResetEmailAPIView(generics.GenericAPIView):
    serializer_class = ResetPasswordEmailRequestSerializer

    def post(self, request):
        nome = (
            request.META.get('HTTP_ORIGIN', '')
            .split('.')[0]
            .upper()
            .split('/')[-1]
        )

        serializer = self.serializer_class(data=request.data)
        email = request.data.get('email', '')

        if User.objects.filter(email=email).exists():
            user = User.objects.get(email=email)

            uidb64 = urlsafe_base64_encode(
                smart_bytes(user.id)
            )
            token = PasswordResetTokenGenerator().make_token(user)

            relative_link = reverse(
                'password-reset-confirm',
                kwargs={
                    'uidb64': uidb64,
                    'token': token
                }
            )

            redirect_url = request.data.get('redirect_url', '')

            base_url = request.META.get(
                'HTTP_ORIGIN',
                'http://mws.mytech.co.mz'
            )

            reset_link = (
                f'{base_url}/#/resetpassword'
                f'{relative_link}?redirect_url={redirect_url}'
            )

            html_message = render_email_template(
                'PASSWORD_RESET',
                {
                    'link': reset_link,
                    'username': user.username,
                    'logo': 'logo',
                }
            )

            try:
                mail = EmailMultiAlternatives(
                    subject=Translate.tdc(
                        request,
                        'Redefinição de palavra-passe'
                    ),
                    body='',
                    to=[email],
                )
                mail.attach_alternative(
                    html_message,
                    'text/html'
                )
                mail.send()

            except Exception:
                return Response(
                    {
                        'alert_error': Translate.tdc(
                            request,
                            'Erro ao enviar o email'
                        )
                    },
                    status=status.HTTP_500_INTERNAL_SERVER_ERROR
                )

        return Response(
            {
                'alert_success': Translate.tdc(
                    request,
                    'Enviámos um link para redefinir a sua palavra-passe'
                )
            },
            status=status.HTTP_200_OK
        )
